// ====================================================================
// Copyright (c) 2002-2006 CeBotics.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           HardwareAPI.h
// Origin Date:    7/9/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains common structures, enumerations, and function
//    export definitions for building HouseBot plugin Hardware
//    Interfaces.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         Eng   Description
// ---------------------------------------
// 1.00     07/09/2002   swl   Initial Version
// 1.01     12/02/2002   swl   Added HBDisplayHelp() export definition.
// 1.02     12/26/2002   swl   Added HBQueryInstance() export definition.
//                             Added hfRequireCommandConfiguration Hardware flag.
// 1.03     02/26/2003   swl    Modified the following methods to pass pointer to
//                               allocated memory instead of using memory returned from HB.
//                               pHB_GetIRCodes()
//                               pHB_GetRemoteControlList()
// 1.04     12/04/2003   swl   Added HBAboutBox() export definition.
// 2.00     01/27/2004   swl   Added m_szModuleWizardDescription to the ModuleInfo struct.
//                             Added m_szWizardFinalText to the ModuleInfo struct.
//                             Added m_bHasPropertiesToBeSetupWithWizard to the ModuleInfo struct.
//                             Added pHB_RegisterHardwareModuleProperty2() and obsoleted pHB_RegisterHardwareModuleProperty().
//                             Added pHB_DoesDeviceExist()
//                             Added pHB_CreateDevice()
//                             Added pHB_SetDeviceProperty()
//                             Added pHB_SetPropertyLimits()
// 3.00     11/09/2006   swl   Added HBInterface() common export.
// 3.01     06/18/2007   swl   Added pHB_NotifySubscribedDevices2() export that replaces pHB_NotifySubscribedDevices()
// 3.02     02/03/2008   swl   Added pHB_DoesDeviceExist2() export that replaces pHB_DoesDeviceExist().
// 										Added HBServerInitialized() export
// ====================================================================

#pragma once

#define HOUSEBOT_HARDWARE_API __declspec(dllexport)

typedef void* HARDWARE_INSTANCE_HANDLE;
typedef void* PROPERTY_VALUE_HANDLE;

//
// Trace message types
enum TRC_TYPE
{
   ttError,       // Error message
   ttWarning,     // Warning message
   ttDebug        // Debug message
};

//
// Property Types
enum P_TYPE
{
   ptList,        // List displayed in a combo-box.
   ptReserved,
   ptButton,      // Do not use.  Only used internally.
   ptToggle,      // Simple two value property.
   ptString,      // Single line text value displayed in an edit field.
   ptInt,         // Integer value.
   ptUnknown
};


//
// Hardware Interface return codes
enum HARDWARE_RC
{
   hrcFailureRetry,  // Call has failed, but you should probably retry.
   hrcFailureAbort,  // Call has failed, please don't retry.
   hrcSuccess        // Call has succeeded.
};


//
// IR Code definition
struct IRCode
{
   int               m_nCodeID;                    // Unique IR code ID.  This must be unique within the entire Remote Control Library.
   const char*       m_szDescription;              // Description of the code.  This will be what the user assigns in the property value mapping.
   const char*       m_szRemoteDescription;        // Description of remote.  This is like a foreign key to the RemoteControl use to identify the Codes owner.
   const char*       m_szInterfaceSpecificData;    // String of proprietary data that can be stored and retrieved by the hardware interface.
   const char*       m_szProntoData;               // Pronto data (if available) for the IR Code.
};

//
// Remote Control definition
struct RemoteControl
{
   int               m_nFrequency;                 // Frequency of the remote control.
   const char*       m_szDescription;              // Unique description for the remote
   const char*       m_szModel;                    // Model number of the remote or device.
   const char*       m_szManufacturer;             // Manufacturer of the remote
};

//
// Module Information structure
struct ModuleInfo
{
   int      m_nModuleInfoSize;                     // Initialize to sizeof( ModuleInfo )
   char*    m_szModuleVersion;                     // The plugin version number
   char*    m_szModuleName;                        // The name of the plugin
   char*    m_szModuleWizardDescription;           // Description of the plugin.  Used in Wizard.
   char*    m_szWizardFinalText;                   // Text displayed at the end of the Wizard process.
   BOOL     m_bHasPropertiesToBeSetupWithWizard;   // Set to TRUE if in HBRegisterProperties() there are properties configured that will be included in the wizard.
};

//
// Hardware flag bit mask values
enum eHardwareFlags
{
   hfNoFlags                      = 0x00,
   hfRequireIRConfiguration       = 0x01,          // Indicates that the plugin has IR configuration.  This causes the server to display the IR mapping
                                                   // property page when configuring property values.
   hfRequireCommandConfiguration  = 0x02           // Indicates that the plugin has general Command configuration.  This causes the server to display
                                                   // the Command mapping page when configuring property values
};


//
// Interface information.
struct InterfaceInfo
{
   int            m_nInterfaceInfoSize;            // Initialize to sizeof( InterfaceInfo )
   char*          m_szInterfaceID;                 // Unique signature that is used by a Device to request services.
   char*          m_szInterfaceName;               // Name of exported function that handles the interface requests.
   char*          m_szInterfaceDescription;        // Description that is shown in the server for the interface.
   eHardwareFlags m_hfFlags;                       // Additional flag values.
};


class CDataPack;
struct InterfaceArgumentPack;

//
// This structure must match the struct defined in ServerInterface.h and HardwareModule.h
struct CallBackInfo
{
   //
   // Current version of the HouseBot Server.
   char*                   m_szHBVersion;

   //
   // This function is OBSOLETE.  It has been superceded by the function pHB_RegisterHardwareModuleProperty2() below.
   BOOL (*pHB_RegisterHardwareModuleProperty)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, P_TYPE pType, const char** aValues, const char* szDefaultValue, BOOL bShowInMainConfiguration );

   //
   // This function will add a button to the Hardware Interface configuration screen for the instance specified.
   BOOL (*pHB_RegisterHardwareModuleButton)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, int nButtonCallbackID, const char* szButtonHeading, const char* szButtonText );
   // Parameters:
   // nButtonCallbackID : Instanced defined ID for the button.  Must be unique for the instance.  Will be returned as a reference if the user presses the button.
   // szButtonHeading : Text that will be displayed over the button in the server Hardware Interface configuration screen.
   // szButtonText : Text that will be displayed on the button in the server Hardware Intervace configuration screen.

   //
   // This function will return the name of a property given a property handle.
   const char* (*pHB_GetPropertyNameFromHandle)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Specifies that handle of the property value that the name is to be returned.

   //
   // This function will return the property value of a property given a name for the specified instance.
   const char* (*pHB_GetPropertyValueFromName)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName );
   // Parameters:
   // szPropertyName : Name of the property value to return the name for.

   //
   // This function will return the property value for the property specified.
   const char* (*pHB_GetPropertyValueFromHandle)( PROPERTY_VALUE_HANDLE hPropertyValue );
   // Parameters:
   // hPropertyValue : Specifies the handle of the property value that the values is to be returned.

   //
   // This function will return the property name and value at the specified index (0 based).  This function can
   // be used to iterate through all properties.  It will return FALSE if the index is invalid.
   BOOL (*pHB_GetPropertyInfoAtIndex)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, int nIndex, char* szPropertyName, char* szPropertyValue );
   // Parameters:
   // nIndex : Index value of property to get.
   // szPropertyName : Return buffer for property name.  Make sure the buffer is large enough to hold the entire buffer contents.
   // szPropertyValue : Return buffer for property value.  Make sure the buffer is large enough to hold the entire buffer contents.

   //
   // This function will set(change) the specified property value.
   BOOL (*pHB_SetPropertyValueFromName)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, const char* szPropertyValue );
   // Parameters:
   // szPropertyName : Name of the property value to change.
   // szPropertyValue : New value for the property.

   //
   // This function will set(change) the specified property name AND value.  It uses an index value
   // to reference the property.
   BOOL (*pHB_SetPropertyNameAndValueAtIndex)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, int nIndex, const char* szPropertyName, const char* szPropertyValue );
   // Parameters:
   // nIndex : 0 based index of the property to change.
   // szPropertyName : New name for the property.  Do NOT pass NULL.  If the name remains the same, pass the old name.
   // szPropertyValue : New value for the property.  Do NOT pass NULL.  If the value remains the same, pass the old value.

   //
   // This function will delete the property specified.
   BOOL (*pHB_DeletePropertyValueByNameAndValue)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, const char* szPropertyValue );
   // Parameters:
   // szPropertyName : Name of the property to delete.
   // szPropertyValue : Value that must match the current value for the property specified by szPropertyName.

   //
   // This function will delete the property specified by an index value.
   BOOL (*pHB_DeletePropertyValueByIndex)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, int nIndex );
   // Parameters:
   // nIndex : 0 based index of the property to delete.

   //
   // This function will trace a message on the servers trace log.
   void (*pHB_TraceMessage)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, TRC_TYPE trcType, const char* szMessage );
   // Parameters:
   // trcType : See values for TRC_TYPE above for possible values.
   // szMessage : Message to trace.

   //
   // This function is OBSOLETE.  It has been superceded by the function pHB_NotifySubscribedDevices2() below.
   void (*pHB_NotifySubscribedDevices)( LPCTSTR szListName, LPCTSTR szFilter, CDataPack* pDataPack );

   //
   // This function will return an array of IR codes from the Remote Control Library.  This function
   // returns TRUE if successful and FALSE if not successful.
   BOOL (*pHB_GetIRCodes)( IRCode* paCodes, DWORD* pdwBufferSize );
   // Parameters:
   // paCodes       : Pointer to buffer to hold array of IRCodes.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in paCodes.  Upon return, this value
   //                   will contain the size of paCodes.

   //
   // This function will return a single IR code from the specified index(position).
   // This function returns TRUE if successful and FALSE if not successful.
   BOOL (*pHB_GetIRCodeFromPosition)( unsigned int usIRPosition, IRCode* pCode );
   // Parameters:
   // usIRPosition : 0 based index into the IR code array.
   // pCode        : Pointer to an IRCode structure that will return the IRCode.

   //
   // This function will return TRUE if the Remote control name supplied is unique or FALSE if
   // the name currently exits.
   BOOL (*pHB_IsRemoteControlDescriptionUnique)( const char* szRemoteDescription );
   // Parameters:
   // szRemoteDescription : Name of the remote control to test for uniqueness.

   //
   // This function will return an array of RemoteControl structures that include all remotes
   // defined in the Remote Control Library.  If successful, the function will return TRUE.
   // If not successful, the function will return FALSE.
   BOOL (*pHB_GetRemoteControlList)( RemoteControl* paRetRemotes, DWORD* pdwBufferSize );
   // Parameters:
   // paRetRemotes  : Pointer to buffer to hold array of RemoteControl.  If this parameter is NULL, the function
   //                   will return the size in bytes needed in pdwBufferSize.
   // pdwBufferSize : Pointer to the size of the buffer passed in paRetRemotes.  Upon return, this value
   //                   will contain the size of paRetRemotes.

   //
   // This function will add a new Remote Control to the Remote Control Library.
   BOOL (*pHB_AddRemoteControl)( const char* szRemoteDescription, int nFrequency, const char* szManufacturer, const char* szModel );
   // Parameters:
   // szRemoteDescription : Name of the new remote control.  The name must be unique.
   // nFrequency : Frequency of the remote
   // szManufacturer : Name of the Remote Manufacturer
   // szModel : Model of the remote.

   //
   // This function will either add a new IR code or update an existing code.
   BOOL (*pHB_AddOrUpdateIRCode)( IRCode* pCode, BOOL bNewCode );
   // Parameters:
   // pCode : IRCode structure (see definition above) that contains IR code information.  If adding a new code,
   //          set pCode->m_nCodeID to -1 to have HouseBot assign the next available IR Code ID value.
   // bNewCode : If set to TRUE, will add the IR code.  If set to FALSE, will modify the IR code.

   //
   // This function will remove the IR code specified.
   BOOL (*pHB_RemoveIRCode)( IRCode* pCode );
   // Parameters:
   // pCode : IR code to be removed.

   //
   // This function will add a property value to the servers database for the instance specified.
   BOOL (*pHB_RegisterHardwareModuleProperty2)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, P_TYPE pType, const char** aValues, const char* szDefaultValue, BOOL bShowInMainConfiguration, const char* szWizardText );
   // Parameters:
   // szPropertyName : Name of Property.  Must be unique for the instance.
   // pType : Type of property.  See P_TYPE enum above for valid values.
   // aValues : Character array of value values that are displayed in lists.  This should be non-NULL for any list type control.
   // szDefaultValue : Default/Current value of the property.
   // bShowInMainConfiguration : If TRUE, the property value will be selectable by the user.  If FALSE, the value will not be shown in the user interface.
   // szWizardText : If not NULL, the Wizard will prompt for a value for this property when creating the new Interface.  It will use the text
   //                specified in the wizard property prompt page.  If NULL, the Wizard will not prompt for a value for the property.

   //
   // This function is OBSOLETE.  It has been superceded by the function pHB_DoesDeviceExist2() below.
   int (*pHB_DoesDeviceExist)( const char* szDeviceName, BOOL bExactMatch, const char* szDeviceDescription, const char* szPropertyName, const char* szPropertyValue );

   //
   // This function will create a new Device in HouseBot.  It will return the new Device ID if successful.
   // If not successful, it will return 0.
   int (*pHB_CreateDevice)( const char* szDeviceName, const char* szDeviceDescription, const char* szDeviceComment, HARDWARE_INSTANCE_HANDLE hInstance );
   // Parameters:
   // szDeviceName : Name of Device.  In HouseBot terms, this is the 'Type' of Device.  This value is required.
   // szDeviceDescription : Description of the Device.  In HouseBot terms, this is actually the 'Name' of the Device and is also required.
   // szDeviceComment : Comment for the new Device.  Can be NULL if no comment is needed.
   // hInstance : If the Device is to be associated with a Hardware Interface, this is the Instance handle to the interface.  If it is
   //             a Software Device, this value should be NULL.

   //
   // This function will set the specified Device Property.
   // If successful, it will return TRUE.  If not successful, it will return FALSE.
   BOOL (*pHB_SetDeviceProperty)( int nDeviceID, const char* szPropertyName, const char* szPropertyValue );
   // Parameters:
   // nDeviceID : Device ID to set the Property for.  This ID can be returned from DoesDeviceExist() or CreateDevice().
   // szPropertyName : The Property name to be set.
   // szPropertyValue : This is the new Property Value to set.  It should be in the form of a configuration string.

   //
   // This function will set the high and low limits for INTEGER property value types.
   BOOL (*pHB_SetPropertyLimits)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, int nLowLimit, int nHighLimit );
   // Parameters:
   // szPropertyName : New name for the property.  Do NOT pass NULL.  If the name remains the same, pass the old name.
   // nLowLimit : The lowest value allowed for the property value.
   // nHighLimit : The highest value allowed for the property value.

   //
   // This function will notify any devices that are subscribed to the specified list.
   void (*pHB_NotifySubscribedDevices2)( HARDWARE_INSTANCE_HANDLE hInstanceHandle, LPCTSTR szListName, LPCTSTR szFilter, CDataPack* pDataPack );
   // Parameters:
   // szListName : "Well Known" list name that is used to provide notification events to interested devices.
   //                Some existing list names are:
   //                "Display Setting Change"
   //                "X10 Reception"
   //                "X10 Reception (ALL)"
   //                "Input State Change"
   //                "X10 Global Reception"
   //                "X10 Global Reception (ALL)"
   //                "IR Response Received"
   //                "Ring Information Received"
   //                "Caller ID Information Received"
   // szFilter : Additional information to help Devices filter their subscription.  For example, if an X10 device only
   //             required notifications when a particular House Code or Unit Code changed, it would subscribe with a
   //             filter value of "H=h;U=u" where h and u represent house code and unit code.  If there is no filter to
   //             specify, NULL can be passed.
   // pDataPack : Data that describes the event.  See Plugins\Common\DataPack.h for the CDataPack class definition.


   //
   // This function will check to see if the specified Device exists.
	// If it does exist, it will return the Device ID and szActualDeviceDescription will contain the description of the device
   // If it does not exist, it will return 0.
   // This function is OBSOLETE.  It has been superceded by the function pHB_DoesDeviceExist2() below.
   int (*pHB_DoesDeviceExist2)( const char* szDeviceName, BOOL bExactMatch, const char* szDeviceDescription, const char* szPropertyName, const char* szPropertyValue, char** szActualDeviceDescription );
   // Parameters:
   // szDeviceName : Name of Device.  In HouseBot terms, this is the 'Type' of Device.  This value is required.
   // bExactMatch : If TRUE, the szDeviceName must macth exactly.  If FALSE, the search will only look at the start of the Device Name when comparing.
   // szDeviceDescription : Description of the Device.  In HouseBot terms, this is actually the 'Name' of the Device that is
   //                         set in the UI.  It can be changed by the user, so it is difficult to test for.  If this parameter is NULL,
   //                         the device description will NOT be used when checking for the exsitance.
   // szPropertyName : To further filter the search, a Property Name can be specified. If this parameter is NULL,
   //                      the search does not look for a specific property name.
   // szPropertyValue : If szPropertyName is specified, this value must specify a specific value for the property.  If
   //                      szPropertyName is NOT specified, this value should be NULL.
	// szActualDeviceDescription : If the device exists, this will return the actual description of the device that was found.  If it does
	// 										not exist, it will return NULL.  This parameter can be NULL, if the caller is not interested.
};

//
// These are the standard functions that are to be exported by every plugin.
extern "C"
{
   //
   // Called ONCE per DLL.  Not called on a per-instance basis.
   HOUSEBOT_HARDWARE_API BOOL HBModuleInit( CallBackInfo* pCallBackInfo, ModuleInfo** ppModuleInfo );

   //
   // Called ONCE for each interface supported by the DLL
   HOUSEBOT_HARDWARE_API BOOL HBHardwareGetInterfaceInfo( int nInterfaceNumber, InterfaceInfo** ppInterfaceInfo );

   //
   // Called ONCE per Instance.
   HOUSEBOT_HARDWARE_API BOOL HBRegisterProperties( HARDWARE_INSTANCE_HANDLE hInstanceHandle );
   HOUSEBOT_HARDWARE_API BOOL HBHardwareInstanceInit( HARDWARE_INSTANCE_HANDLE hInstanceHandle );
   HOUSEBOT_HARDWARE_API BOOL HBHardwareInstanceTerminate( HARDWARE_INSTANCE_HANDLE hInstanceHandle );

   //
   // Called when needed.
   HOUSEBOT_HARDWARE_API BOOL HBUpdatePropertyValue( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szPropertyName, const char* szNewValue );
   HOUSEBOT_HARDWARE_API BOOL HBPropertyButtonClick( HARDWARE_INSTANCE_HANDLE hInstanceHandle, int nButtonCallbackID );
   HOUSEBOT_HARDWARE_API void HBDisplayHelp();
   HOUSEBOT_HARDWARE_API BOOL HBQueryInstance( HARDWARE_INSTANCE_HANDLE hInstanceHandle, const char* szQuery, void* pResponse );
   HOUSEBOT_HARDWARE_API void HBAboutBox();
   HOUSEBOT_HARDWARE_API void HBServerInitialized();

	//
	// Optional export that can be used as a single call/export instead of unique exports for every interface.
   HOUSEBOT_HARDWARE_API HARDWARE_RC HBInterface( HARDWARE_INSTANCE_HANDLE hInstanceHandle, InterfaceArgumentPack* pPack );
}
