// ====================================================================
// Copyright (c) 2002-2006 CeBotics.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           HardwareInstance.cpp
// Origin Date:    9/4/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains the class implementation for CHardwareInterface.
//    The CHardwareInterface class is a helper class for writing HouseBot
//    Hardware plugin Interfaces.  The class wraps the server callback
//    function pointers for easier calling.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         Eng   Description
// ---------------------------------------
// 1.00     09/04/2002   swl   Initial Version
// 1.01     12/02/2002   swl   Added CheckForServerVersion() method.
// 1.02     02/26/2003   swl    Modified the following methods to pass pointer to
//                               allocated memory instead of using memory returned from HB.
//                               GetIRCodes()
//                               GetRemoteControlList()
// 1.03     05/07/03     swl   Improved TraceMessage to be more tolerant of longer messages.
// 2.00     01/27/04     swl   Added wizard text parameter to RegisterHardwareModuleProperty() method
// 3.01     06/18/07     swl   Changed NotifySubscribedDevices() to use the new pHB_NotifySubscribedDevices2() callback
// 3.02     02/03/2008   swl   Added szActualDeviceDescription paramter to DoesDeviceExist to return the description of the device if it DOES exist.
// ====================================================================

#include "..\..\stdafx.h"
#include "HardwareInstance.h"
#include "..\..\Common\InterfaceAPI.h"

// ====================================================================
// Method Name : CHardwareInstance
// Parameters  : hInstance - Hardware Instance Handle passed by HBRegisterProperties()
//                pCallBackInfo - Copy of callback info structure passed by HBModuleInit()
// Description : Constructor for the HardwareInterface helper class.  Derive hardware instances
//                from this class to help manage server callbacks.
// ====================================================================
CHardwareInstance::CHardwareInstance( HARDWARE_INSTANCE_HANDLE hInstance, CallBackInfo* pCallBackInfo ) :
   m_hInstance( hInstance ),
   m_pCallBackInfo( pCallBackInfo )
{
}


// ====================================================================
// Method Name : CharToDump
// Description : This method will format character data into a hexadecimal
//                formatted output string.
// Parameters  : szData - character data to show in hexadecimal format
//               nDataLen - length of szData
//               szReturnBuffer - character buffer to hold the returned hexadecimal output
//               nReturnBufferLen - length of szReturnBuffer
// Returns     : szReturnBuffer
// ====================================================================
const char* CHardwareInstance::CharToDump( const unsigned char* szData, int nDataLen, char* szReturnBuffer, int nReturnBufferLen )
{
   //
   // If 0 or -1 was passed as the data length, we can assume that this is a NULL terminated
   // string and calculate the length.
   if ((nDataLen == -1) || (nDataLen == 0))
   {
      nDataLen = strlen( (char*)szData );
   }

   memset( szReturnBuffer, 0, nReturnBufferLen );

   //nDataLen = min( nDataLen, nReturnBufferLen / 3 );

   if (nDataLen)
   {
      char  szHex[ 5 ] = {0};
      int   nSrc;
      int   nDest = 0;
      for (nSrc = 0; nSrc < nDataLen; nSrc++)
      {
         //
         // Make sure there is enough room for the next block
         if (nDest + 3 >= nReturnBufferLen)
         {
            break;
         }

         wsprintf( szHex, "%02x ", (int)szData[ nSrc ] );
         strcat( szReturnBuffer, szHex );

         nDest += 3;
      }

      //
      // Remove the final space
      szReturnBuffer[ strlen( szReturnBuffer ) -1 ] = '\0';
   }

   return( szReturnBuffer );
}


// ====================================================================
// Method Name : GetLastErrorAsString
// Description : Display error message text, given an error code.
//                Typically, the parameter passed to this function is retrieved
//                from GetLastError().
// Returns     : Pointer to static buffer containing the error.  This method is
//                not thread safe, since the return buffer is shared by all threads.
// ====================================================================
char* CHardwareInstance::GetLastErrorAsString()
{
   static char  szErrorBuffer[ 512 ] = {0};

   FormatMessage(
     FORMAT_MESSAGE_FROM_SYSTEM,
     NULL,
     GetLastError(),
     0,
     szErrorBuffer,
     511,
     NULL );

   return( szErrorBuffer );
}

// ====================================================================
// Method Name : TextFromInterfaceArgument
// Description : This method will return a character string from the supplied
//                InterfaceArgument data.
// Parameters  : pArgument - InterfaceArgument data to be converted to character string.
// Returns     : Character string representing the value passed in pArgument.
//                Returns "* Error *" if there was an error parsing the value
// ====================================================================
const char* CHardwareInstance::TextFromInterfaceArgument( InterfaceArgument* pArgument )
{
   if (pArgument)
   {
      switch (pArgument->m_atType)
      {
         case atPropertyValue:
            return( GetPropertyValueFromHandle( pArgument->m_pvhPropertyValue ) );

         case atStringValue:
            return( pArgument->m_szStringValue );

         case atIntValue:           // Not supported
         case atVoidValue:          // Not supported
         default:
            break;
      }
   }

   return( "* Error *" );
}

// ====================================================================
// Method Name : IntFromInterfaceArgument
// Description : This method will return an integer from the supplied
//                InterfaceArgument data.
// Parameters  : pArgument - InterfaceArgument data to be converted to an integer
// Returns     : Integer representing the value passed in pArgument.
//                Returns 0 if there was an error parsing the value
// ====================================================================
int CHardwareInstance::IntFromInterfaceArgument( InterfaceArgument* pArgument )
{
   if (pArgument)
   {
      switch (pArgument->m_atType)
      {
         case atPropertyValue:
            return( atoi(GetPropertyValueFromHandle( pArgument->m_pvhPropertyValue )) );

         case atStringValue:
            return( atoi( pArgument->m_szStringValue ));

         case atIntValue:
               return( pArgument->m_nIntValue );

         case atVoidValue:          // Not supported
         default:
            break;
      }
   }

   return( 0 );
}

// ====================================================================
// Method Name : VoidFromInterfaceArgument
// Description : This method will return a void pointer from the supplied
//                InterfaceArgument data.
// Parameters  : pArgument - InterfaceArgument data
// Returns     : Void pointer data stored in InterfaceArgument.
//                Returns NULL if there was an error parsing the value, or
//                if the value was not a void data type.
// ====================================================================
void* CHardwareInstance::VoidFromInterfaceArgument( InterfaceArgument* pArgument )
{
   if (pArgument)
   {
      switch (pArgument->m_atType)
      {
         case atVoidValue:
            return( pArgument->m_pVoidValue );


         case atPropertyValue:      // Not supported
         case atStringValue:        // Not supported
         case atIntValue:           // Not supported

         default:
            break;
      }
   }

   return( NULL );
}


// ====================================================================
// Method Name : CheckForServerVersion
// Description : This method will check to see if the server version is at
//                or above the version values passed.
// Parameters  : nMajor - Integer that specifies the required major version number
//             : nMinor - Integer that specifies the required minor version number
//             : nSubMinor - Integer that specifies the required subminor version number.
// Returns     : TRUE if version is >= values passed.  FALSE if version is < values passed.
// ====================================================================
BOOL CHardwareInstance::CheckForServerVersion( int nMajor, int nMinor, int nSubMinor )
{
   int   nMaj = 0;
   int   nMin = 0;
   int   nSubMin = 0;

   if ((!m_pCallBackInfo) ||
       (!m_pCallBackInfo->m_szHBVersion))
   {
      return( FALSE );
   }

   if (3 != sscanf( m_pCallBackInfo->m_szHBVersion, "%d.%d.%d", &nMaj, &nMin, &nSubMin ))
   {
      if (2 != sscanf( m_pCallBackInfo->m_szHBVersion, "%d.%d", &nMaj, &nMin ))
      {
         return( FALSE );
      }
   }

   //
   // Test the values.
   if (nMaj > nMajor)
   {
      return( TRUE );
   }

   if ((nMaj == nMajor) && (nMin > nMinor))
   {
      return( TRUE );
   }

   if ((nMaj == nMajor) && (nMin == nMinor) && (nSubMin >= nSubMinor))
   {
      return( TRUE );
   }

   return( FALSE );
}

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// Call back helper wrappers.  See HardwareAPI.h for description
// of the methods and their parameters.
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
BOOL CHardwareInstance::RegisterHardwareModuleProperty( const char* szPropertyName, P_TYPE pType, const char** aValues, const char* szDefaultValue, BOOL bShowInMainConfiguration, const char* szWizardText )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_RegisterHardwareModuleProperty2( m_hInstance, szPropertyName, pType, aValues, szDefaultValue, bShowInMainConfiguration, szWizardText ));
}

BOOL CHardwareInstance::SetPropertyLimits( const char* szPropertyName, int nLowLimit, int nHighLimit )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_SetPropertyLimits( m_hInstance, szPropertyName, nLowLimit, nHighLimit ));
}

BOOL CHardwareInstance::RegisterHardwareModuleButton( int nButtonCallbackID, const char* szButtonHeading, const char* szButtonText )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_RegisterHardwareModuleButton( m_hInstance, nButtonCallbackID, szButtonHeading, szButtonText ) );
}

const char* CHardwareInstance::GetPropertyNameFromHandle( PROPERTY_VALUE_HANDLE hPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetPropertyNameFromHandle( hPropertyValue )  );
}

const char* CHardwareInstance::GetPropertyValueFromName( const char* szPropertyName )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetPropertyValueFromName( m_hInstance, szPropertyName )  );
}

const char* CHardwareInstance::GetPropertyValueFromHandle( PROPERTY_VALUE_HANDLE hPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetPropertyValueFromHandle( hPropertyValue )  );
}

BOOL CHardwareInstance::GetPropertyInfoAtIndex( int nIndex, char* szPropertyName, char* szPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetPropertyInfoAtIndex( m_hInstance, nIndex, szPropertyName, szPropertyValue )  );
}

BOOL CHardwareInstance::SetPropertyValueFromName( const char* szPropertyName, const char* szPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_SetPropertyValueFromName( m_hInstance, szPropertyName, szPropertyValue )  );
}

BOOL CHardwareInstance::SetPropertyNameAndValueAtIndex( int nIndex, const char* szPropertyName, const char* szPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_SetPropertyNameAndValueAtIndex( m_hInstance, nIndex, szPropertyName, szPropertyValue )  );
}

BOOL CHardwareInstance::DeletePropertyValueByNameAndValue( const char* szPropertyName, const char* szPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_DeletePropertyValueByNameAndValue( m_hInstance, szPropertyName, szPropertyValue )  );
}

BOOL CHardwareInstance::DeletePropertyValueByIndex( int nIndex )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_DeletePropertyValueByIndex( m_hInstance, nIndex )  );
}

void CHardwareInstance::TraceMessage( TRC_TYPE ttType, const char* szMessage, ... )
{
   char  szFormattedMessage[ 1024 ];

   //
   // Format the message
   va_list args;
   va_start( args, szMessage );
   int nLen = _vsnprintf( szFormattedMessage, 1023, szMessage, args );
   if (nLen == -1)
   {
      nLen = sizeof( szFormattedMessage ) - 1;
   }
   szFormattedMessage[ nLen ] = 0;

   //
   // Send it to the server.
   CMFCStateSwitcher switcher;
   m_pCallBackInfo->pHB_TraceMessage( m_hInstance, ttType, szFormattedMessage );
}

void CHardwareInstance::NotifySubscribedDevices( LPCTSTR szListName, LPCTSTR szFilter, CDataPack* pDataPack )
{
	if (m_pCallBackInfo->pHB_NotifySubscribedDevices2 != NULL)
	{
		TraceMessage( ttDebug, "Sending Subscription Notification for list [%s]", szListName );

		CMFCStateSwitcher switcher;
		m_pCallBackInfo->pHB_NotifySubscribedDevices2( m_hInstance, szListName, szFilter, pDataPack );
	}
	else
	{
		TraceMessage( ttError, "Unable to send subscription notification.  Your version of HouseBot does not support this function.  Consider upgrading." );
	}
}

BOOL CHardwareInstance::GetIRCodes( IRCode* paCodes, DWORD* pdwBufferSize )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetIRCodes( paCodes, pdwBufferSize ) );
}

BOOL CHardwareInstance::GetIRCodeFromPosition( unsigned int usIRPosition, IRCode* pCode )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetIRCodeFromPosition( usIRPosition, pCode )  );
}

BOOL CHardwareInstance::IsRemoteControlDescriptionUnique( const char* szRemoteDescription )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_IsRemoteControlDescriptionUnique( szRemoteDescription )  );
}

BOOL CHardwareInstance::GetRemoteControlList( RemoteControl* paRetRemotes, DWORD* pdwBufferSize )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_GetRemoteControlList( paRetRemotes, pdwBufferSize ) );
}

BOOL CHardwareInstance::AddRemoteControl( const char* szRemoteDescription, int nFrequency, const char* szManufacturer, const char* szModel )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_AddRemoteControl( szRemoteDescription, nFrequency, szManufacturer, szModel )  );
}

BOOL CHardwareInstance::AddOrUpdateIRCode( IRCode* pCode, BOOL bNewCode )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_AddOrUpdateIRCode( pCode, bNewCode )  );
}

BOOL CHardwareInstance::RemoveIRCode( IRCode* pCode )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_RemoveIRCode( pCode )  );
}

int CHardwareInstance::DoesDeviceExist( const char* szDeviceName, BOOL bExactMatch, const char* szDeviceDescription, const char* szPropertyName, const char* szPropertyValue, char** szActualDeviceDescription )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_DoesDeviceExist2( szDeviceName, bExactMatch, szDeviceDescription, szPropertyName, szPropertyValue, szActualDeviceDescription ) );
}

int CHardwareInstance::CreateDevice( const char* szDeviceName, const char* szDeviceDescription, const char* szDeviceComment, HARDWARE_INSTANCE_HANDLE hInstance )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_CreateDevice( szDeviceName, szDeviceDescription, szDeviceComment, hInstance ) );
}

BOOL CHardwareInstance::SetDeviceProperty( int nDeviceID, const char* szPropertyName, const char* szPropertyValue )
{
   CMFCStateSwitcher switcher;
   return( m_pCallBackInfo->pHB_SetDeviceProperty( nDeviceID, szPropertyName, szPropertyValue ) );
}

