// ====================================================================
// Copyright (c) 2002-2007 CeBotics LLC.  All rights reserved.
//    This file is provided as part of a software development kit (SDK).
//    This file can be used, royalty free, in any form to extend the
//    functionality of the HouseBot product.
//
// File:           HardwareInstance.h
// Origin Date:    9/4/2002
// Platform:       Developed using MS VC++ 6.0
// Purpose:
//    This file contains the class definition for CHardwareInterface.
//    The CHardwareInterface class is a helper class for writing HouseBot
//    Hardware plugin Interfaces.  The class wraps the server callback
//    function pointers for easier calling.
// ====================================================================

// ====================================================================
// Version Information
// Ver      Date         Eng   Description
// ---------------------------------------
// 1.00     09/04/2002   swl   Initial Version
// 1.01     12/02/2002   swl   Added CheckForServerVersion() method.
// 1.02     02/26/2003   swl    Modified the following methods to pass pointer to
//                               allocated memory instead of using memory returned from HB.
//                               GetIRCodes()
//                               GetRemoteControlList()
// 2.00     02/20/2004   swl   Added SetDeviceProperty(), CreateDevice(), DoesDeviceExist(), and SetPropertyLimits() methods.
// 3.00     01/01/2007   swl	 Added HBInterface() method.
// 3.01     06/18/2007   swl   Changed NotifySubscribedDevices() to use the new pHB_NotifySubscribedDevices2() callback
// 3.02     02/03/2008   swl   Added szActualDeviceDescription paramter to DoesDeviceExist to return the description of the device if it DOES exist.
// ====================================================================

#pragma once

//
// Disable warning about debug symbols
#pragma warning(disable:4786)
#include <map>

#include "HardwareAPI.h"

//
// Forward definitions
struct InterfaceArgument;
struct InterfaceArgumentPack;
class CDataPack;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// CHardwareInstance
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
class CHardwareInstance
{
public:
   virtual ~CHardwareInstance() {};

   //
   // Derived classes have the option of deriving these methods.  These
   // methods map to exports that are called by the server.
   virtual BOOL Init() { return( TRUE ); }
   virtual void Terminate() {};
   virtual BOOL RegisterProperties() { return( TRUE ); }
   virtual BOOL UpdatePropertyValue( const char* szPropertyName, const char* szNewValue ) { return( TRUE ); }
   virtual BOOL PropertyButtonClick( int nButtonCallbackID ) { return( TRUE ); }
   virtual HARDWARE_RC HBInterface( const char* szInterfaceName, InterfaceArgumentPack* pPack   ) { return( hrcFailureAbort ); }

   inline HARDWARE_INSTANCE_HANDLE GetInstanceHandle() { return( m_hInstance ); }

   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   // Call back helper wrappers.  See HardwareAPI.h for description
   // of the methods and their parameters.
   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   BOOL           RegisterHardwareModuleProperty( const char* szPropertyName, P_TYPE pType, const char** aValues, const char* szDefaultValue, BOOL bShowInMainConfiguration, const char* szWizardText = NULL );
   BOOL           SetPropertyLimits( const char* szPropertyName, int nLowLimit, int nHighLimit );
   BOOL           RegisterHardwareModuleButton( int nButtonCallbackID, const char* szButtonHeading, const char* szButtonText );
   const char*    GetPropertyNameFromHandle( PROPERTY_VALUE_HANDLE hPropertyValue );
   const char*    GetPropertyValueFromName( const char* szPropertyName );
   const char*    GetPropertyValueFromHandle( PROPERTY_VALUE_HANDLE hPropertyValue );
   BOOL           GetPropertyInfoAtIndex( int nIndex, char* szPropertyName, char* szPropertyValue );
   BOOL           SetPropertyValueFromName( const char* szPropertyName, const char* szPropertyValue );
   BOOL           SetPropertyNameAndValueAtIndex( int nIndex, const char* szPropertyName, const char* szPropertyValue );
   BOOL           DeletePropertyValueByNameAndValue( const char* szPropertyName, const char* szPropertyValue );
   BOOL           DeletePropertyValueByIndex( int nIndex );
   void           NotifySubscribedDevices( LPCTSTR szListName, LPCTSTR szFilter, CDataPack* pDataPack );
   BOOL           GetIRCodes( IRCode* paCodes, DWORD* pdwBufferSize );
   BOOL           GetIRCodeFromPosition( unsigned int usIRPosition, IRCode* pCode );
   BOOL           AddOrUpdateIRCode( IRCode* pCode, BOOL bNewCode );
   BOOL           RemoveIRCode( IRCode* pCode );
   BOOL           IsRemoteControlDescriptionUnique( const char* szRemoteDescription );
   BOOL           GetRemoteControlList( RemoteControl* paRetRemotes, DWORD* pdwBufferSize );
   BOOL           AddRemoteControl( const char* szRemoteDescription, int nFrequency, const char* szManufacturer, const char* szModel );
   void           TraceMessage( TRC_TYPE ttType, const char* szMessage, ... );


   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   // Other helper methods
   ///////////////////////////////////////////////////////////////////////
   ///////////////////////////////////////////////////////////////////////
   const char*    CharToDump( const unsigned char* szData, int nDataLen, char* szReturnBuffer, int nReturnBufferLen );
   char*          GetLastErrorAsString();

   int            DoesDeviceExist( const char* szDeviceName, BOOL bExactMatch, const char* szDeviceDescription, const char* szPropertyName, const char* szPropertyValue, char** szActualDeviceDescription = NULL );
   int            CreateDevice( const char* szDeviceName, const char* szDeviceDescription, const char* szDeviceComment, HARDWARE_INSTANCE_HANDLE hInstance );
   BOOL           SetDeviceProperty( int nDeviceID, const char* szPropertyName, const char* szPropertyValue );

protected:
   CHardwareInstance( HARDWARE_INSTANCE_HANDLE hInstance, CallBackInfo* pCallBackInfo );

   //
   // Helper functions to parse InterfaceArgumentPack arguments.
   const char*    TextFromInterfaceArgument( InterfaceArgument* pArgument );
   int            IntFromInterfaceArgument( InterfaceArgument* pArgument );
   void*          VoidFromInterfaceArgument( InterfaceArgument* pArgument );

   BOOL           CheckForServerVersion( int nMajor, int nMinor, int nSubMinor );


   HARDWARE_INSTANCE_HANDLE   m_hInstance;
   CallBackInfo*              m_pCallBackInfo;
};


// ====================================================================
// Class Name  : CMFCStateSwitcher
// Description : This class helps to manage the MFC thread/module
//                state of MFC DLLs.  It is used by the CHardwareInstance
//                class before calling back into the server.
// ====================================================================
#ifdef __AFX_H__
class CMFCStateSwitcher
{
public:
   CMFCStateSwitcher() { m_pDLLModuleState = AfxSetModuleState( AfxGetThreadState()->m_pPrevModuleState );  }
   ~CMFCStateSwitcher() { AfxSetModuleState( m_pDLLModuleState ); }

   AFX_MODULE_STATE*    m_pDLLModuleState;
};

#else
class CMFCStateSwitcher
{
public:
   CMFCStateSwitcher() {};
};

#endif
